import matplotlib.pyplot as plt
import numpy as np
#from mpl_toolkits.mplot3d import Axes3D

def PlotValOld(Th,u):
  if Th.d==2:
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot_trisurf(Th.q[:,0],Th.q[:,1],u,triangles=Th.me, cmap=plt.cm.Spectral)
    plt.show()
   
    
def PlotIsolines(Th,u,**kwargs):
  if Th.d==2:
    N=kwargs.get('N', 15 )
    Fill=kwargs.get('fill', False )
    Colorbar=kwargs.get('colorbar', False )
    fig = plt.gcf()
    plt.gca().set_aspect('equal')
    if Fill:
      #plt.tricontourf(Th.q[:,0],Th.q[:,1],Th.me, u,N,shading='interp')
      plt.tripcolor(Th.q[:,0],Th.q[:,1],Th.me, u, shading='gouraud')#, cmap=plt.cm.rainbow)
    else:
      plt.tricontour(Th.q[:,0],Th.q[:,1],Th.me, u,N)
    if Colorbar:
      plt.colorbar()
    plt.show()
  
def PlotMesh(Th):
  if Th.d==2:
    fig = plt.gcf()
    plt.triplot(Th.q[:,0],Th.q[:,1],triangles=Th.me)
    plt.gca().set_aspect('equal')
    plt.show()
    
  
def PlotBounds(Th,**kwargs):
  assert(Th.d==2)
  linewidth=kwargs.get('linewidth', 2.0 )
  legend=kwargs.get('legend', True )
  fontsize=kwargs.get('fontsize', 16 )
  Color=kwargs.get('color', None )
  LB=np.unique(Th.bel)
  fig = plt.gcf()
  plt.rc('text', usetex=True)
  ax = fig.gca()
  ax.axis('equal')
  Lines=[]
  for i in range(len(LB)):
    I,=np.where(Th.bel==LB[i])
    line,=ax.plot(np.r_[Th.q[Th.be[I[0],0],0],Th.q[Th.be[I[0],1],0]],np.r_[Th.q[Th.be[I[0],0],1],Th.q[Th.be[I[0],1],1]],linewidth=linewidth)
      
    Lines.append(line)
    line.set_label(r"$\Gamma_{"+str(int(LB[i]))+"}$")
    
  for i in range(len(LB)):
    I,=np.where(Th.bel==LB[i])
    if Color==None:
      color=Lines[i].get_color()
    else:
      color=Color
   
    for k in I:
      line,=ax.plot(np.r_[Th.q[Th.be[k,0],0],Th.q[Th.be[k,1],0]],np.r_[Th.q[Th.be[k,0],1],Th.q[Th.be[k,1],1]],color=color,linewidth=linewidth)
    
  box = ax.get_position()
  ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
  #ax.legend(loc='best')
  if legend:
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5),fancybox=True, shadow=True,fontsize=fontsize)
  plt.show()
  
  
def PlotVal(Th,x,**kwargs):
  assert(Th.d==2)
  N=kwargs.get('N', 15 )
  colorbar=kwargs.get('colorbar', True )
  plt.hold(True)
  PlotBounds(Th,legend=False,color='k')  
#  plt.axis('off')
  PlotIsolines(Th,x,fill=True,colorbar=colorbar)
  PlotIsolines(Th,x,N=N)
  
def showSparsity(M):
#  from matplotlib.pyplot as plt
  plt.spy(M, precision=1e-8, marker='.', markersize=3)
  plt.show()